@@ -94,7 +94,7 @@ class ShortcutDialog(QDialog):
 
 
 class ShortcutName(QDialog):
-    def __init__(self, texts, default=u'', parent=None):
+    def __init__(self, texts, default='', parent=None):
         super(ShortcutName, self).__init__(parent)
         self.setWindowTitle('puddletag')
         self.ok = False
@@ -596,6 +596,10 @@ class CreateAction(QDialog):
         self.buttonlist.duplicate.connect(self.duplicate)
         self.listbox.currentRowChanged.connect(self.enableEditButtons)
         self.listbox.itemDoubleClicked.connect(self.edit)
+        
+        if len(self.functions) == 0:
+            self.buttonlist.duplicateButton.setEnabled(False)
+            self.buttonlist.editButton.setEnabled(False)
 
         if prevfunctions is not None:
             self.functions = copy(prevfunctions)
