@@ -1,24 +1,27 @@
 # -*- coding: utf-8 -*-
-
-import pdb
-import audioinfo, os, string, re
-from decimal import Decimal
-from pyparsing import (Word, alphas,Literal, OneOrMore, alphanums, 
-    nums, delimitedList, Combine, QuotedString, 
-    CharsNotIn, originalTextFor, nestedExpr, 
-    Optional)
-from puddleobjects import PuddleConfig, safe_name
-from funcprint import pprint
-from puddlestuff.util import PluginFunction, translate, to_list, to_string
-import cPickle as pickle
-stringtags = audioinfo.stringtags
-from copy import deepcopy
-from constants import ACTIONDIR, CHECKBOX, SPINBOX, SYNTAX_ERROR, SYNTAX_ARG_ERROR
 import glob
-from collections import defaultdict
-from functools import partial
-numtimes = 0
+import os
+import pickle
+import re
 import string
+from collections import defaultdict
+from copy import deepcopy
+from decimal import Decimal
+from functools import partial
+
+from pyparsing import (Word, alphas, Literal, OneOrMore, alphanums,
+                       nums, delimitedList, Combine, QuotedString,
+                       CharsNotIn, originalTextFor, nestedExpr,
+                       Optional)
+
+from . import audioinfo
+from .constants import ACTIONDIR, CHECKBOX, SPINBOX, SYNTAX_ERROR, SYNTAX_ARG_ERROR
+from .funcprint import pprint
+from .puddleobjects import PuddleConfig, safe_name
+from .util import PluginFunction, translate, to_list, to_string
+
+numtimes = 0
+stringtags = audioinfo.stringtags
 
 NOT_ALL = audioinfo.INFOTAGS + ['__image']
 FILETAGS = audioinfo.FILETAGS
@@ -28,37 +31,42 @@ FUNC_MODULE = 'module'
 ARGS = 'arguments'
 KEYWORD_ARGS = set(['tags', 'm_tags', 'r_tags', 'state'])
 
-whitespace = set(unicode(string.whitespace))
+whitespace = set(str(string.whitespace))
+
 
 class ParseError(Exception):
     def __init__(self, message):
         Exception.__init__(self, message)
         self.message = message
 
+
 class FuncError(ParseError): pass
 
+
 class MultiValueError(FuncError): pass
 
-from functions import functions
 
-def arglen_error(e, passed, function, to_raise = True):
-    varnames = function.func_code.co_varnames[:function.func_code.co_argcount]
+from .functions import functions
+
+
+def arglen_error(e, passed, function, to_raise=True):
+    varnames = function.__code__.co_varnames[:function.__code__.co_argcount]
     args_len = len(passed)
     param_len = len(varnames)
     message = None
     if args_len > param_len:
         message = translate('Functions',
-            'At most %1 arguments expected. %2 given.')
+                            'At most %1 arguments expected. %2 given.')
     elif args_len < param_len:
-        default_len = len(function.func_defaults) if \
-            function.func_defaults else 0
+        default_len = len(function.__defaults__) if \
+            function.__defaults__ else 0
         if args_len < (param_len - default_len):
             message = translate('Functions',
-            'At least %1 arguments expected. %2 given.')
+                                'At least %1 arguments expected. %2 given.')
     else:
         raise e
     if message is not None:
-        message = message.arg(unicode(param_len)).arg(unicode(args_len))
+        message = message.arg(str(param_len)).arg(str(args_len))
     else:
         raise e
     if to_raise:
@@ -66,6 +74,7 @@ def arglen_error(e, passed, function, to_raise = True):
     else:
         return message
 
+
 def convert_actions(dirpath, new_dir):
     backup = os.path.join(dirpath, 'actions.bak')
     if not os.path.exists(backup):
@@ -79,7 +88,8 @@ def convert_actions(dirpath, new_dir):
         os.rename(filename, path_join(backup, basename(filename)))
         save_macro(path_join(new_dir, basename(filename)), name, funcs)
 
-def filenametotag(pattern, filename, checkext = False, split_dirs=True):
+
+def filenametotag(pattern, filename, checkext=False, split_dirs=True):
     """Retrieves tag values from your filename
         pattern is the rule with which to extract
         the tags from filename. Which does not have to
@@ -110,9 +120,9 @@ def filenametotag(pattern, filename, checkext = False, split_dirs=True):
         filename = os.path.splitext(filename)[0]
 
     e = Combine(Literal("%").suppress() + OneOrMore(Word(alphas)) + Literal("%").suppress())
-    patterns = filter(None, pattern.split(u'/'))
+    patterns = [_f for _f in pattern.split('/') if _f]
     if split_dirs:
-        filenames = filename.split(u'/')[-len(patterns):]
+        filenames = filename.split('/')[-len(patterns):]
     else:
         filenames = [filename]
     mydict = {}
@@ -126,11 +136,12 @@ def filenametotag(pattern, filename, checkext = False, split_dirs=True):
             else:
                 mydict[key] = new_fields[key]
     if mydict:
-        if mydict.has_key("dummy"):
-            del(mydict["dummy"])
+        if "dummy" in mydict:
+            del (mydict["dummy"])
         return mydict
     return {}
 
+
 def get_old_action(filename):
     """Gets the action from filename, where filename is either a string or
     file-like object.
@@ -139,7 +150,7 @@ def get_old_action(filename):
     these are stored as pickled objects.
 
     Returns [list of Function objects, action name]."""
-    if isinstance(filename, basestring):
+    if isinstance(filename, str):
         f = open(filename, "rb")
     else:
         f = filename
@@ -148,6 +159,7 @@ def get_old_action(filename):
     f.close()
     return [funcs, name]
 
+
 def load_macro_info(filename):
     modules = defaultdict(lambda: defaultdict(lambda: {}))
     for function in functions.values():
@@ -158,20 +170,17 @@ def load_macro_info(filename):
             modules[function.__module__][function.__name__] = function
     cparser = PuddleConfig(filename)
     funcs = []
-    name = cparser.get('info', 'name', u'')
+    name = cparser.get('info', 'name', '')
 
-    def get_func_index(funcname):
-        if funcname.startswith('Func'):
-            return int(funcname[4:].strip())
-        else:
-            return funcname
+    key_type = lambda text: int(text) if text.isdigit() else text.lower()
+    get_func_index = lambda key: [ key_type(c) for c in re.split('([0-9]+)', key) ]
 
     for section in sorted(cparser.sections(), key=get_func_index):
-        if section.startswith(u'Func'):
+        if section.startswith('Func'):
             get = partial(cparser.get, section)
-            func_name = get(FUNC_NAME, u'')
+            func_name = get(FUNC_NAME, '')
             fields = get(FIELDS, [])
-            func_module = get(FUNC_MODULE, u'')
+            func_module = get(FUNC_MODULE, '')
             arguments = get(ARGS, [])
             try:
                 func = Function(modules[func_module][func_name], fields)
@@ -182,7 +191,7 @@ def load_macro_info(filename):
             newargs = []
             for i, (control, arg) in enumerate(zip(func.controls, arguments)):
                 if control == CHECKBOX:
-                    if arg == False or arg == u'False':
+                    if arg == False or arg == 'False':
                         newargs.append(False)
                     else:
                         newargs.append(True)
@@ -195,14 +204,16 @@ def load_macro_info(filename):
             else:
                 func.args = arguments
             funcs.append(func)
-    return [funcs, name]    
+    return [funcs, name]
+
 
 def load_macro_from_name(name):
     filename = os.path.join(ACTIONDIR, safe_name(name) + '.action')
     return Macro(filename)
 
+
 def func_tokens(dictionary, parse_action):
-    func_name = Word(alphas+'_', alphanums+'_')
+    func_name = Word(alphas + '_', alphanums + '_')
 
     func_ident = Combine('$' + func_name.copy()('funcname'))
     func_tok = func_ident + originalTextFor(nestedExpr())('args')
@@ -214,12 +225,12 @@ def func_tokens(dictionary, parse_action):
 
     def replace_token(tokens):
         index = int(tokens.num)
-        return dictionary.get(index, u'')
+        return dictionary.get(index, '')
 
     rx_tok.setParseAction(replace_token)
 
     strip = lambda s, l, tok: tok[0].strip()
-    text_tok = CharsNotIn(u',').setParseAction(strip)
+    text_tok = CharsNotIn(',').setParseAction(strip)
     quote_tok = QuotedString('"')
 
     if dictionary:
@@ -229,21 +240,21 @@ def func_tokens(dictionary, parse_action):
 
     return func_tok, arglist, rx_tok
 
+
 def get_function_arguments(funcname, func, arguments, reserved, fmt=True, *dicts):
+    varnames = func.__code__.co_varnames[:func.__code__.co_argcount]
 
-    varnames = func.func_code.co_varnames[:func.func_code.co_argcount]
+    # arguments will contain only a list of user supplied arguments
+    # Eg. for the function $format(%artist%) the user will specify
+    # only the %artist%, whereas calling the function requires
+    # the tags in order to look it up.
+    # This, and below replaces the tags with their corresponding order.
 
-    #arguments will contain only a list of user supplied arguments
-    #Eg. for the function $format(%artist%) the user will specify
-    #only the %artist%, whereas calling the function requires
-    #the tags in order to look it up.
-    #This, and below replaces the tags with their corresponding order.
-    
     topass = {}
     othervars = []
 
     key_args = set(KEYWORD_ARGS).union(reserved)
-    
+
     for i, v in enumerate(varnames):
         if v in key_args:
             topass[v] = reserved[v]
@@ -260,15 +271,16 @@ def get_function_arguments(funcname, func, arguments, reserved, fmt=True, *dicts
             except ValueError:
                 raise ParseError(SYNTAX_ARG_ERROR % (funcname, no + 1))
         else:
-            if isinstance(arg, basestring) and fmt:
+            if isinstance(arg, str) and fmt:
                 topass[param] = replacevars(arg, *dicts)
             else:
                 topass[param] = arg
 
     return topass
 
+
 def run_format_func(funcname, arguments, m_audio, s_audio=None, extra=None,
-    state=None):
+                    state=None):
     '''Runs the function function using the arguments specified from pudlestuff.function.
 
     Arguments:
@@ -277,7 +289,7 @@ def run_format_func(funcname, arguments, m_audio, s_audio=None, extra=None,
     arguments -- List of arguments to pass to the function. Patterns
                  should not be evaluated. They'll be evaluated here.
     m_audio   -- Audio file containg multiple multiple values per key.
-                 Eg. {'artist': [u'Artist1': 'Artist2']}
+                 Eg. {'artist': ['Artist1': 'Artist2']}
 
     Keyword Arguments
     s_audio -- Same as m_audio, but containing strings as values.
@@ -288,10 +300,10 @@ def run_format_func(funcname, arguments, m_audio, s_audio=None, extra=None,
              Used by some functions in puddlestuff.functions
     
     '''
-    
-    #Get function
+
+    # Get function
     try:
-        if isinstance(funcname, basestring):
+        if isinstance(funcname, str):
             func = functions[funcname]
         else:
             func = funcname
@@ -301,25 +313,25 @@ def run_format_func(funcname, arguments, m_audio, s_audio=None, extra=None,
 
     extra = {} if extra is None else extra
     s_audio = stringtags(m_audio) if s_audio is None else s_audio
-    
+
     reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state}
     dicts = [s_audio, extra, state]
-    topass = get_function_arguments(funcname, func, arguments, 
-        reserved, True, *dicts)
+    topass = get_function_arguments(funcname, func, arguments, reserved, True, *dicts)
 
     try:
         ret = func(**topass)
         if ret is None:
-            return u''
+            return ''
         return ret
-    except TypeError, e:
+    except TypeError as e:
         message = SYNTAX_ERROR.arg(funcname)
         message = message.arg(arglen_error(e, topass, func, False))
         raise ParseError(message)
-    except FuncError, e:
+    except FuncError as e:
         message = SYNTAX_ERROR.arg(funcname).arg(e.message)
         raise ParseError(message)
 
+
 def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, path_sep=None):
     """Parses format strings. Returns the parsed string.
 
@@ -327,7 +339,7 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
     ---------
     s  -- *Unicode* format string. Eg. $replace(%artist%, name, surname)
     m_audio -- Audio file containg multiple multiple values per key.
-        Eg. {'artist': [u'Artist1': 'Artist2']}
+        Eg. {'artist': ['Artist1': 'Artist2']}
 
     Keyword Arguments
     s_audio -- Same as m_audio, but containing strings as values.
@@ -337,65 +349,64 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
     state -- Dictionary that hold state. Like {'__count': 15}.
              Used by some functions in puddlestuff.functions
 
-    >>> audio = {'artist': [u'Artist1'], 'track':u'10'}
-    >>> parsefunc(u'%track% - %artist%', audio)
+    >>> audio = {'artist': ['Artist1'], 'track':'10'}
+    >>> parsefunc('%track% - %artist%', audio)
     Artist1 - 10
-    >>> state = {'__count': u'5'}
-    >>> parsefunc(u'$num(%track%, 2)/$num(%__count%, 2). %artist%', audio,
+    >>> state = {'__count': '5'}
+    >>> parsefunc('$num(%track%, 2)/$num(%__count%, 2). %artist%', audio,
     ... state = state)
-    u'05/05. Artist1'
+    '05/05. Artist1'
 
     """
 
-    #Yes I know this is a big ass function...but it works and I can't
-    #see a way to split it without making it complicated.
-    
+    # Yes I know this is a big ass function...but it works and I can't
+    # see a way to split it without making it complicated.
+
     tokens = []
     token = []
-    #List containing a function with it's arguments
-    #Will look like ['replace', arg1, arg2, arg3]
-    #functions get evaluated as soon as closing bracket found.
+    # List containing a function with it's arguments
+    # Will look like ['replace', arg1, arg2, arg3]
+    # functions get evaluated as soon as closing bracket found.
     func = []
-    #Flag determining if current within a function. Used for making comma's
-    #significant
+    # Flag determining if current within a function. Used for making comma's
+    # significant
     in_func = False
     in_quote = False
-    #field_open == -1 if not current in the middle of processing a field
-    #eg. %artist%. Otherwise contains the index in s that the field started.
+    # field_open == -1 if not current in the middle of processing a field
+    # eg. %artist%. Otherwise contains the index in s that the field started.
     field_open = -1
-    #Determine if next char should be escaped.
+    # Determine if next char should be escaped.
     escape = False
-        
-    
+
     s_audio = stringtags(m_audio) if s_audio is None else s_audio
     state = {} if state is None else state
     tags = s_audio.copy()
     tags.update(state)
     tags.update(extra if extra is not None else {})
+
     escape_chars = set('()$%\\,')
 
     br_error = translate('Errors', 'No closing bracket found.')
 
     paths = []
 
-    
     i = 0
     while 1:
         try:
             c = s[i]
-        except IndexError:  #  Parsing's done.
+        except IndexError:  # Parsing's done.
             if in_func:
                 raise ParseError(SYNTAX_ERROR.arg(func[0]).arg(br_error))
             if token:
-                tokens.append(replacevars(u''.join(token), tags))
+                tokens.append(replacevars(''.join(token), tags))
             break
-        
-        if c == u'"' and not escape:
+
+        if c == '"' and not escape:
             if in_func:
                 token.append(c)
             in_quote = not in_quote
-        elif c == u'"' and escape and in_func:
-            token.append(u'\\"')
+        elif c == '"' and escape and in_func:
+            token.append('\\"')
             i += 1
             escape = False
             continue
@@ -406,7 +417,7 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
             continue
         elif in_quote:
             token.append(c)
-        elif c == u'\\' and not escape:
+        elif c == '\\' and not escape:
             escape = True
             i += 1
             try:
@@ -418,8 +429,8 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
                 token.append(c)
                 escape = False
             continue
-        elif c == u'$' and not (escape or (field_open >= 0)):
-            func_name = re.search(u'^\$(\w+)\(', s[i:])
+        elif c == '$' and not (escape or (field_open >= 0)):
+            func_name = re.search('^\$(\w+)\(', s[i:])
             if not func_name:
                 token.append(c)
                 i += 1
@@ -431,7 +442,7 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
                 i += offset + 1
                 continue
 
-            tokens.append(replacevars(u''.join(token), tags))
+            tokens.append(replacevars(''.join(token), tags))
             token = []
             func = []
             func_name = func_name.groups(0)[0]
@@ -440,14 +451,15 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
             i += len(func_name) + 1
         elif in_func and not in_quote and not token and c in whitespace:
             'just increment counter'
-        elif c == u',' and in_func and not in_quote:
-            func.append(u''.join(token))
+        elif c == ',' and in_func and not in_quote:
+            func.append(''.join(token))
             token = []
-        elif c == u')' and in_func:
+        elif c == ')' and in_func:
             in_func = False
-            if token or s[i-1] == u',':
-                func.append(u''.join(token))
-            func_parsed = run_format_func(func[0], func[1:], m_audio, s_audio)
+            if token or s[i - 1] == ',':
+                func.append(''.join(token))
+
+            func_parsed = run_format_func(func[0], func[1:], m_audio, s_audio, state=state, extra=extra)
             if ret_i:
                 return func_parsed, i
             tokens.append(func_parsed)
@@ -455,14 +467,15 @@ def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, pat
         else:
             token.append(c)
             if path_sep and c == path_sep and not in_func:
-                paths.append(len(u''.join(tokens) + replacevars(u''.join(token), tags)) - 1)
+                paths.append(len(''.join(tokens) + replacevars(''.join(token), tags)) - 1)
         escape = False
         i += 1
 
     if path_sep and not ret_i:
-        return paths, u''.join(tokens)
+        return paths, ''.join(tokens)
     else:
-        return u''.join(tokens)
+        return ''.join(tokens)
+
 
 def parse_field_list(fields, audio, selected=None):
     fields = fields[::]
@@ -482,7 +495,7 @@ def parse_field_list(fields, audio, selected=None):
                 not_fields.remove('__selected')
         fields = fields[:index]
         fields.extend([key for key in audio if key not in
-            not_fields and key not in NOT_ALL])
+                       not_fields and key not in NOT_ALL])
 
     if '__all' in fields:
         while '__all' in fields:
@@ -496,20 +509,25 @@ def parse_field_list(fields, audio, selected=None):
             fields.extend(selected)
     return list(set(fields))
 
+
 # This function is from name2id3 by  Kristian Kvilekval
 def re_escape(rex):
     """Escape regular expression special characters"""
     escaped = ""
     for ch in rex:
-        if ch in r'^$[]\+*?.(){},|' : escaped = escaped + '\\' + ch
-        else: escaped = escaped + ch
+        if ch in r'^$[]\+*?.(){},|':
+            escaped = escaped + '\\' + ch
+        else:
+            escaped = escaped + ch
     return escaped
 
+
 def removeSpaces(text):
     for char in string.whitespace:
         text = text.replace(char, '')
     return text.lower()
 
+
 def replacevars(pattern, *dicts):
     """Replaces occurrences of %key% with the d[key] in the string pattern.
 
@@ -526,15 +544,15 @@ def replacevars(pattern, *dicts):
     Returns the parsed string. If a key isn't found, but in the format
     string, it'll be removed.
 
-    >>> replacevars('%artist%', {'artist': u'ARTIST'})
-    u'ARTIST'
-    >>> replacevars('one %two%', {"three": u'VALUE'})
-    u'one '
+    >>> replacevars('%artist%', {'artist': 'ARTIST'})
+    'ARTIST'
+    >>> replacevars('one %two%', {"three": 'VALUE'})
+    'one '
 
     """
     r_vars = {}
-    map(r_vars.update, [z for z in dicts if z])
-    
+    list(map(r_vars.update, [z for z in dicts if z]))
+
     in_quote = False
     in_field = False
     ret = []
@@ -546,28 +564,28 @@ def replacevars(pattern, *dicts):
             next_char = pattern[i + 1]
         except IndexError:
             next_char = None
-        if c == u'\\' and next_char == '"' and not escape:
+        if c == '\\' and next_char == '"' and not escape:
             escape = True
             continue
         elif escape:
             escape = False
-        elif c == u'"':
+        elif c == '"':
             in_quote = not in_quote
             continue
-        elif c == u'%' and not in_quote:
+        elif c == '%' and not in_quote:
 
             if not in_field:
                 field_start = len(ret)
                 in_field = True
             elif in_field:
                 in_field = False
-                field = u''.join(ret[field_start:])
-                del(ret[field_start:])
-                ret.append(r_vars.get(field, u''))
+                field = ''.join(ret[field_start:])
+                del (ret[field_start:])
+                ret.append(r_vars.get(field, ''))
             continue
         ret.append(c)
 
-    return u''.join(ret)
+    return ''.join(ret)
 
 
 def apply_actions(actions, audio, state=None, ovr_fields=None):
@@ -575,15 +593,15 @@ def apply_actions(actions, audio, state=None, ovr_fields=None):
 
     actions can be a list of Function objects or a filename of an action file (see load_action).
     audio must dictionary-like object."""
-    
+
     if state is None:
         state = {}
     if '__counter' not in state:
         state['__counter'] = 0
-    state['__counter'] = unicode(int(state['__counter']) + 1)
+    state['__counter'] = str(int(state['__counter']) + 1)
 
     r_tags = audio
-   
+
     if hasattr(audio, 'tags'):
         audio = deepcopy(audio.tags)
     else:
@@ -598,11 +616,11 @@ def apply_actions(actions, audio, state=None, ovr_fields=None):
         ret = {}
 
         for field in fields:
-            val = audio.get(field, u'')
+            val = audio.get(field, '')
             temp = func.runFunction(val, audio, state, None, r_tags)
             if temp is None:
                 continue
-            if isinstance(temp, basestring):
+            if isinstance(temp, str):
                 ret[field] = temp
             elif hasattr(temp, 'items'):
                 ret.update(temp)
@@ -612,7 +630,7 @@ def apply_actions(actions, audio, state=None, ovr_fields=None):
             elif hasattr(temp[0], 'items'):
                 [ret.update(z) for z in temp]
                 break
-            elif isinstance(temp[0], basestring):
+            elif isinstance(temp[0], str):
                 if field in FILETAGS:
                     ret[field] = temp[0]
                 else:
@@ -625,6 +643,7 @@ def apply_actions(actions, audio, state=None, ovr_fields=None):
             audio.update(ret)
     return dict([(z, audio[z]) for z in changed])
 
+
 def apply_macros(macros, audio, state, fields=None):
     actions = []
     for m in macros:
@@ -634,13 +653,14 @@ def apply_macros(macros, audio, state, fields=None):
         return apply_actions(actions, audio, state, fields)
     else:
         return apply_actions(actions, audio, state)
-    
+
 
 def runQuickAction(funcs, audio, state, tag):
     """Same as runAction, except that all funcs are 
     applied not in the values stored but on audio[tag]."""
     return apply_macros(funcs, audio, state, tag)
 
+
 def save_macro(filename, name, funcs):
     f = open(filename, 'w')
     f.close()
@@ -653,17 +673,19 @@ def save_macro(filename, name, funcs):
         set_value(i, FUNC_MODULE, func.function.__module__)
         set_value(i, ARGS, func.args)
 
+
 def saveAction(filename, actionname, funcs):
     """Saves an action to filename.
 
     funcs is a list of funcs, and actionname is...er...the name of the action."""
-    if isinstance(filename, basestring):
+    if isinstance(filename, str):
         fileobj = open(filename, 'wb')
     else:
         fileobj = filename
     pickle.dump(actionname, fileobj)
     pickle.dump(funcs, fileobj)
 
+
 def tagtofilename(pattern, filename, addext=False, extension=None, state=None):
     """
     tagtofilename sets the filename of an mp3 or ogg file
@@ -708,9 +730,9 @@ def tagtofilename(pattern, filename, addext=False, extension=None, state=None):
     >>>tagtofilename(pattern, filename, True, '.mp3')
     Amy Winehouse - 012 - Shitty Song.mp3"""
 
-    #First check if a filename was passed or a dictionary.
-    if not isinstance(filename, basestring):
-        #if it was a dictionary, then use that as the tags.
+    # First check if a filename was passed or a dictionary.
+    if not isinstance(filename, str):
+        # if it was a dictionary, then use that as the tags.
         tags = filename
     else:
         tags = audioinfo.Tag(filename)
@@ -724,6 +746,7 @@ def tagtofilename(pattern, filename, addext=False, extension=None, state=None):
     else:
         return parsefunc(pattern, tags, state=state) + os.path.extsep + tags["__ext"]
 
+
 def tagtotag(pattern, text, expression):
     """See filenametotag for an implementation example and explanation.
 
@@ -736,6 +759,7 @@ def tagtotag(pattern, text, expression):
     """
     pattern = re_escape(pattern)
     taglist = []
+
     def what(s, loc, tok):
         global numtimes
         taglist.append(tok[0])
@@ -743,6 +767,7 @@ def tagtotag(pattern, text, expression):
         if numtimes == 0:
             return "(.*)"
         return "(.*?)"
+
     expression.setParseAction(what)
     global numtimes
     numtimes = len([z for z in expression.scanString(pattern)])
@@ -752,16 +777,17 @@ def tagtotag(pattern, text, expression):
     try:
         tags = re.search(pattern, text).groups()
     except AttributeError:
-        #No matches were found
-        return  u''
-    mydict={}
+        # No matches were found
+        return ''
+    mydict = {}
     for i in range(len(tags)):
-        if mydict.has_key(taglist[i]):
-            mydict[taglist[i]] = ''.join([mydict[taglist[i]],tags[i]])
+        if taglist[i] in mydict:
+            mydict[taglist[i]] = ''.join([mydict[taglist[i]], tags[i]])
         else:
-            mydict[taglist[i]]=tags[i]
+            mydict[taglist[i]] = tags[i]
     return mydict
 
+
 class Function:
     """Basically, a wrapper for functions, but makes it easier to
     call according to the needs of puddletag.
@@ -778,12 +804,12 @@ class Function:
     See the functions module for more info."""
 
     def __init__(self, funcname, fields=None):
-        if isinstance(funcname, basestring):
+        if isinstance(funcname, str):
             self.function = functions[funcname]
         elif isinstance(funcname, PluginFunction):
             self.function = funcname.function
-            self.doc = [u','.join([funcname.name, funcname.print_string])] + \
-                [','.join(z) for z in funcname.args]
+            self.doc = [','.join([funcname.name, funcname.print_string])] + \
+                       [','.join(z) for z in funcname.args]
             self.info = [funcname.name, funcname.print_string]
         else:
             self.function = funcname
@@ -796,13 +822,13 @@ class Function:
         else:
             self.tag = ''
         self.args = None
-        
+
         self.controls = self._getControls()
 
     def reInit(self):
-        #Since this class gets pickled in ActionWindow, the class is never 'destroyed'
-        #Since, a functions docstring wouldn't be reflected back to puddletag
-        #if it were changed calling this function to 're-read' it is a good idea.
+        # Since this class gets pickled in ActionWindow, the class is never 'destroyed'
+        # Since, a functions docstring wouldn't be reflected back to puddletag
+        # if it were changed calling this function to 're-read' it is a good idea.
         if not self.function.__doc__:
             return
         self.doc = self.function.__doc__.split("\n")
@@ -815,16 +841,16 @@ class Function:
     def setArgs(self, args):
         self.args = args
 
-    def runFunction (self, text=None, m_tags=None, state=None,
-        tags=None, r_tags=None):
+    def runFunction(self, text=None, m_tags=None, state=None,
+                    tags=None, r_tags=None):
 
         func = self.function
 
-        varnames = func.func_code.co_varnames[:func.func_code.co_argcount]
+        varnames = func.__code__.co_varnames[:func.__code__.co_argcount]
 
         if not varnames:
             return func()
-            
+
         s_audio = stringtags(m_tags) if tags is None else tags
         m_audio = m_tags
         state = {} if state is None else state
@@ -833,21 +859,21 @@ class Function:
         text = to_string(text)
 
         reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
-            'r_tags': r_tags, 'm_text': m_text}
-        
+                    'r_tags': r_tags, 'm_text': m_text}
+
         if varnames[0] in reserved:
             reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
-                'r_tags': r_tags, 'text': to_string(text),
-                'm_text': m_text}
+                        'r_tags': r_tags, 'text': to_string(text),
+                        'm_text': m_text}
             topass = get_function_arguments("", func, self.args, reserved,
-                False, *[s_audio, state])
+                                            False, *[s_audio, state])
             return func(**topass)
         else:
             reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
-                'r_tags': r_tags}
+                        'r_tags': r_tags}
             topass = get_function_arguments("", func,
-                [text] + self.args, reserved, False, *[s_audio, state])
-            
+                                            [text] + self.args, reserved, False, *[s_audio, state])
+
         try:
             first_arg = [z for z in varnames if z not in reserved][0]
         except IndexError:
@@ -869,16 +895,16 @@ class Function:
             return func(**topass)
 
     def description(self):
-        d = [u", ".join(self.tag)] + self.args
+        d = [", ".join(self.tag)] + self.args
         return pprint(translate('Functions', self.info[1]), d)
-        
+
     def _getControls(self, index=1):
         identifier = QuotedString('"') | CharsNotIn(',')
         arglist = delimitedList(identifier)
         docstr = self.doc[1:]
         if index:
             return [(arglist.parseString(line)[index]).strip()
-                for line in docstr]
+                    for line in docstr]
         else:
             ret = []
             for line in docstr:
@@ -890,15 +916,16 @@ class Function:
         self.fields = tag
 
     def addArg(self, arg):
-        if self.function.func_code.co_argcount > len(self.args) + 1:
+        if self.function.__code__.co_argcount > len(self.args) + 1:
             self.args.append(arg)
 
+
 class Macro(object):
     def __init__(self, filename=None):
         object.__init__(self)
-        self.name = u''
+        self.name = ''
         self.actions = []
-        self.filename = u''
+        self.filename = ''
         if filename is not None:
             self.load(filename)
 
@@ -918,6 +945,5 @@ class Macro(object):
     def save(self, filename=None):
         if filename is None:
             filename = self.filename
-    
+
         save_macro(filename, self.name, self.actions)
-        
