@@ -1,40 +1,42 @@
 # -*- coding: utf-8 -*-
 """Dialog's that crop up along the application, but are used at at most
 one place, and aren't that complicated are put here."""
-
-import os, pdb, sys
-
+import os
+import sys
 from copy import deepcopy
-from functools import partial
 
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
+from PyQt5.QtCore import QItemSelectionModel, Qt, pyqtRemoveInputHook, pyqtSignal
+from PyQt5.QtGui import QPalette, QBrush, QColor
+from PyQt5.QtWidgets import QAbstractItemView, QAction, QApplication, QCheckBox, QComboBox, QCompleter, \
+    QDialog, QFileDialog, QFrame, QGridLayout, QHBoxLayout, QHeaderView, QLabel, QLineEdit, QMessageBox, \
+    QPlainTextEdit, QPushButton, QSpinBox, QTableWidget, QTableWidgetItem, QTextEdit, QToolButton, QVBoxLayout
 
-from puddlestuff.util import pprint_tag
-import puddlestuff.findfunc as findfunc
-import puddlestuff.resource
-
-from audioinfo import commontags, PATH, FILE_FIELDS
-from puddlestuff.constants import HOMEDIR, KEEP
-from puddleobjects import (
+from . import findfunc, audioinfo
+from .audioinfo import commontags, PATH
+from .constants import HOMEDIR, KEEP
+from .puddleobjects import (
     get_icon, gettaglist, partial,
     settaglist, winsettings, ListButtons, MoveButtons, OKCancel,
-    PicWidget, PuddleConfig, natcasecmp)
-from puddlestuff.translations import translate
-from puddlestuff.util import to_string
+    PicWidget, PuddleConfig, natsort_case_key)
+from .translations import translate
+from .util import pprint_tag
+from .util import to_string
 
 ADD, EDIT, REMOVE = (1, 2, 3)
 UNCHANGED = 0
 BOLD = 1
 ITALICS = 2
-TAG_DISP = u"<b>%s: </b> %s, "
+TAG_DISP = "<b>%s: </b> %s, "
+
 
 class AutonumberDialog(QDialog):
-    def __init__(self, parent=None, minval=0, numtracks = 0,
-        enablenumtracks = False):
-            
-        QDialog.__init__(self,parent)
-        
+    newtracks = pyqtSignal(int, int, 'Qt::CheckState', int, str, str, 'Qt::CheckState', name='newtracks')
+
+    def __init__(self, parent=None, minval=0, numtracks=0,
+                 enablenumtracks=False):
+
+        QDialog.__init__(self, parent)
+
         self.setWindowTitle(
             translate('Autonumbering Wizard', "Autonumbering Wizard"))
         winsettings('autonumbering', self)
@@ -62,22 +64,22 @@ class AutonumberDialog(QDialog):
         self._padlength.setMinimum(1)
 
         label = QLabel(translate('Autonumbering Wizard',
-            'Max length after padding with zeroes: '))
+                                 'Max length after padding with zeroes: '))
         label.setBuddy(self._padlength)
 
         vbox.addLayout(hbox(label, self._padlength))
 
         self._separator = QCheckBox(translate('Autonumbering Wizard',
-            "Add track &separator ['/']: Number of tracks"))
+                                              "Add track &separator ['/']: Number of tracks"))
         self._numtracks = QSpinBox()
         self._numtracks.setEnabled(False)
         self._numtracks.setMaximum(65535)
         if numtracks:
             self._numtracks.setValue(numtracks)
         self._restart_numbering = QCheckBox(translate('Autonumbering Wizard',
-            "&Restart numbering at each directory group."))
-        self.connect(self._restart_numbering, SIGNAL("stateChanged(int)"),
-                     self.showDirectorySplittingOptions)
+                                                      "&Restart numbering at each directory group."))
+        self._restart_numbering.stateChanged.connect(
+            self.showDirectorySplittingOptions)
 
         vbox.addLayout(hbox(self._separator, self._numtracks))
         vbox.addWidget(self._restart_numbering)
@@ -88,27 +90,27 @@ class AutonumberDialog(QDialog):
 
         self.grouping = QLineEdit()
         label.setBuddy(self.grouping)
-        
+
         vbox.addLayout(hbox(label, self.grouping))
         self.custom_numbering_widgets.extend([label, self.grouping])
 
         label = QLabel(translate('Autonumbering Wizard', "Output field: "))
-        
+
         self.output_field = QComboBox()
         label.setBuddy(self.output_field)
-        
+
         self.output_field.setEditable(True)
         completer = self.output_field.completer()
         completer.setCaseSensitivity(Qt.CaseSensitive)
         completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
-        
+
         self.output_field.setCompleter(completer)
         self.output_field.addItems(gettaglist())
         vbox.addLayout(hbox(label, self.output_field))
         self.custom_numbering_widgets.extend([label, self.output_field])
 
-        self.count_by_group = QCheckBox(translate(u'Autonumbering Wizard',
-                                                    u'Increase counter only on group change'))
+        self.count_by_group = QCheckBox(translate('Autonumbering Wizard',
+                                                  'Increase counter only on group change'))
         vbox.addWidget(self.count_by_group)
         self.custom_numbering_widgets.append(self.count_by_group)
 
@@ -116,12 +118,12 @@ class AutonumberDialog(QDialog):
         vbox.addLayout(okcancel)
         self.setLayout(vbox)
 
-        self.connect(okcancel,SIGNAL('ok'), self.emitValuesAndSave)
-        self.connect(okcancel,SIGNAL('cancel'),self.close)
-        self.connect(self._separator, SIGNAL("stateChanged(int)"),
+        okcancel.ok.connect(self.emitValuesAndSave)
+        okcancel.cancel.connect(self.close)
+        self._separator.stateChanged.connect(
             lambda v: self._numtracks.setEnabled(v == Qt.Checked))
-        
-        # self.connect(self._restart_numbering, SIGNAL("stateChanged(int)"),
+
+        # self._restart_numbering.stateChanged.connect(
         #              self.showDirectorySplittingOptions)
 
         self._separator.setChecked(enablenumtracks)
@@ -129,43 +131,43 @@ class AutonumberDialog(QDialog):
         self._loadSettings()
 
     def showDirectorySplittingOptions(self, state):
-        is_checked = state==Qt.Checked
+        is_checked = state == Qt.Checked
         for widget in self.custom_numbering_widgets:
             widget.setVisible(is_checked)
 
         if is_checked:
             self._numtracks.setVisible(False)
             self._separator.setText(translate('Autonumbering Wizard',
-            "Add track &separator ['/']"))
+                                              "Add track &separator ['/']"))
         else:
             self._numtracks.setVisible(True)
             self._separator.setText(translate('Autonumbering Wizard',
-            "Add track &separator ['/']: Number of tracks"))
+                                              "Add track &separator ['/']: Number of tracks"))
 
     def emitValuesAndSave(self):
         if self._numtracks.isVisible():
             if self._separator.isChecked():
                 numtracks = self._numtracks.value()
             else:
-                numtracks = -1 #Don't use totals
+                numtracks = -1  # Don't use totals
         else:
             if self._separator.isChecked():
-                numtracks = -2 #Use totals, automaticall generated
+                numtracks = -2  # Use totals, automaticall generated
             else:
-                numtracks = -1 
+                numtracks = -1
 
         self.close()
-        
-        self.emit(SIGNAL("newtracks"),
-                  self._start.value(),
-                  numtracks,
-                  self._restart_numbering.checkState(),
-                  self._padlength.value(),
-                  unicode(self.grouping.text()),
-                  unicode(self.output_field.currentText()),
-                  self.count_by_group.checkState()
+
+        self.newtracks.emit(
+            self._start.value(),
+            numtracks,
+            self._restart_numbering.checkState(),
+            self._padlength.value(),
+            str(self.grouping.text()),
+            str(self.output_field.currentText()),
+            self.count_by_group.checkState()
         )
-        
+
         self._saveSettings()
 
     def _loadSettings(self):
@@ -175,17 +177,17 @@ class AutonumberDialog(QDialog):
         self._separator.setCheckState(
             cparser.get(section, 'separator', Qt.Unchecked))
         self._padlength.setValue(cparser.get(section, 'padlength', 1))
-        
+
         self._restart_numbering.setCheckState(
             cparser.get(section, 'restart', Qt.Unchecked))
 
         self.count_by_group.setCheckState(
             cparser.get(section, 'count_by_group', Qt.Unchecked))
-        
+
         self.showDirectorySplittingOptions(self._restart_numbering.checkState())
-        
+
         self.grouping.setText(cparser.get(section, 'grouping', '%__dirpath%'))
-        
+
         output_field_text = cparser.get(section, 'output_field', 'track')
         if not output_field_text:
             output_field_text = 'track'
@@ -206,11 +208,14 @@ class AutonumberDialog(QDialog):
         cparser.set(section, 'grouping', self.grouping.text())
         cparser.set(section, 'output_field', self.output_field.currentText())
 
+
 class ImportTextFile(QDialog):
     """Dialog that importing a text file to retrieve tags from."""
-    def __init__(self,parent = None, filename = None, clipboard = None):
+    Newtags = pyqtSignal(list, str, name='Newtags')
+
+    def __init__(self, parent=None, filename=None, clipboard=None):
         QDialog.__init__(self, parent)
-        
+
         self.setWindowTitle(
             translate('Text File -> Tag', "Import tags from text file"))
         winsettings('importwin', self)
@@ -223,12 +228,11 @@ class ImportTextFile(QDialog):
         self.label = QLabel(translate('Text File -> Tag', "Tag preview"))
         grid.addWidget(self.label, 0, 2)
 
-
         self.file = QTextEdit()
         grid.addWidget(self.file, 1, 0, 1, 2)
 
         self.tags = QTextEdit()
-        grid.addWidget(self.tags,1, 2, 1, 2)
+        grid.addWidget(self.tags, 1, 2, 1, 2)
         self.tags.setLineWrapMode(QTextEdit.NoWrap)
 
         hbox = QHBoxLayout()
@@ -238,26 +242,26 @@ class ImportTextFile(QDialog):
         self.patterncombo.setDuplicatesEnabled(False)
 
         okcancel = OKCancel()
-        self.ok = okcancel.ok
-        self.cancel = okcancel.cancel
+        self.ok = okcancel.okButton
+        self.cancel = okcancel.cancelButton
 
         self.openfile = QPushButton(
             translate('Text File -> Tag', "&Select File"))
         getclip = QPushButton(
             translate('Text File -> Tag', "&Paste Clipboard"))
-        self.connect(getclip, SIGNAL('clicked()'), self.openClipBoard)
+        getclip.clicked.connect(self.openClipBoard)
 
         hbox.addWidget(self.openfile)
         hbox.addWidget(getclip)
-        hbox.addWidget(self.patterncombo,1)
+        hbox.addWidget(self.patterncombo, 1)
         hbox.addLayout(okcancel)
 
         grid.addLayout(hbox, 3, 0, 1, 4)
         self.setLayout(grid)
 
-        self.connect(self.openfile,SIGNAL("clicked()"),self.openFile)
-        self.connect(self.cancel, SIGNAL("clicked()"),self.close)
-        self.connect(self.ok, SIGNAL("clicked()"), self.emitValues)
+        self.openfile.clicked.connect(self.openFile)
+        self.cancel.clicked.connect(self.close)
+        self.ok.clicked.connect(self.emitValues)
 
         if clipboard:
             self.openClipBoard()
@@ -271,25 +275,26 @@ class ImportTextFile(QDialog):
     def emitValues(self):
         """When I'm done, emit a signal with the updated tags."""
         self.close()
-        self.emit(SIGNAL("Newtags"), self.dicttags,
-            unicode(self.patterncombo.currentText()))
+        self.Newtags.emit(self.dicttags,
+                          str(self.patterncombo.currentText()))
 
-    def fillTags(self, string = None): #string is there purely for the SIGNAL
+    def fillTags(self, string=None):  # string is there purely for the SIGNAL
         """Fill the tag textbox."""
+
         def formattag(tags):
             if tags:
                 return pprint_tag(tags, TAG_DISP, True)[:-2]
             else:
-                return u""
+                return ""
 
         self.dicttags = []
         self.tags.clear()
-        for z in self.lines.split(u"\n"):
+        for z in self.lines.split("\n"):
             self.dicttags.append(findfunc.filenametotag(
-                unicode(self.patterncombo.currentText()), z, False, False))
+                str(self.patterncombo.currentText()), z, False, False))
         if self.dicttags:
             self.tags.setHtml(
-                u"<br/>".join([formattag(z) for z in self.dicttags]))
+                "<br/>".join([formattag(z) for z in self.dicttags]))
 
     def openFile(self, filename=None, dirpath=None):
         """Open the file and fills the textboxes."""
@@ -297,54 +302,51 @@ class ImportTextFile(QDialog):
             dirpath = self.lastDir
 
         if not filename:
-            filedlg = QFileDialog()
-            filename = unicode(filedlg.getOpenFileName(self,
-                'OpenFolder', dirpath))
+            selectedFile = QFileDialog.getOpenFileName(self,
+                                                       'OpenFolder', dirpath)
+            filename = selectedFile[0]
 
         if not filename:
             return True
 
         try:
             f = open(filename, 'r')
-        except (IOError, OSError), detail:
+        except (IOError, OSError) as detail:
             errormsg = translate('Text File -> Tag',
-                "The file <b>%1</b> couldn't be loaded.<br /> "
-                "Do you want to choose another?")
+                                 "The file <b>%1</b> couldn't be loaded.<br /> "
+                                 "Do you want to choose another?")
 
             ret = QMessageBox.question(self,
-                translate('Text File -> Tag', "Error"),
-                translate('Text File -> Tag', errormsg.arg(filename)),
-                translate('Text File -> Tag', "&Yes"),
-                translate('Text File -> Tag', "&No"))
+                                       translate('Text File -> Tag', "Error"),
+                                       translate('Text File -> Tag', errormsg.arg(filename)))
 
-            if ret == 0:
+            if ret == QMessageBox.Yes:
                 return self.openFile()
             else:
                 return detail
 
-        self.lines = [z.decode('utf8', 'replace') for z in f.readlines()]
-        self.file.setPlainText(u"".join(self.lines))
+        self.lines = f.readlines()
+        self.file.setPlainText("".join(self.lines))
         self.setLines()
         self.fillTags()
         self.show()
-        self.connect(self.file, SIGNAL("textChanged()"), self.setLines)
-        self.connect(self.patterncombo, SIGNAL("editTextChanged(QString)"),
+        self.file.textChanged.connect(self.setLines)
+        self.patterncombo.editTextChanged.connect(
             self.fillTags)
         self.lastDir = os.path.dirname(filename)
 
     def openClipBoard(self):
-        text = unicode(QApplication.clipboard().text())
-        self.lines = text.split(u'\n')
+        text = str(QApplication.clipboard().text())
+        self.lines = text.split('\n')
         self.file.setPlainText(text)
         self.setLines()
         self.fillTags()
         self.show()
-        self.connect(self.file, SIGNAL("textChanged()"), self.setLines)
-        self.connect(self.patterncombo,
-            SIGNAL("editTextChanged(QString)"), self.fillTags)
+        self.file.textChanged.connect(self.setLines)
+        self.patterncombo.editTextChanged.connect(self.fillTags)
 
     def setLines(self):
-        self.lines = unicode(self.file.document().toPlainText())
+        self.lines = str(self.file.document().toPlainText())
         self.fillTags()
 
 
@@ -377,13 +379,14 @@ class EditField(QDialog):
     in the form {field: value}.
     (Because the user might choose to edit a different tag,
     then the one that was chosen and you'd want to delete that one)"""
+    donewithmyshit = pyqtSignal(str, str, object, name='donewithmyshit')
 
     def __init__(self, field=None, parent=None, field_list=None, edit=True):
 
         QDialog.__init__(self, parent)
         self.setWindowTitle(translate('Edit Field', 'Edit Field'))
         winsettings('edit_field', self)
-        
+
         self.vbox = QVBoxLayout()
 
         label = QLabel(translate('Edit Field', "&Field"))
@@ -396,15 +399,15 @@ class EditField(QDialog):
         self.tagcombo.setCompleter(completer)
         self.tagcombo.addItems(field_list if field_list else gettaglist())
 
-        #Get the previous field
+        # Get the previous field
         self.__oldField = field
         label1 = QLabel(translate('Edit Field', "&Value"))
         self.value = TextEdit()
         self.value.setTabChangesFocus(True)
         label1.setBuddy(self.value)
-        
+
         okcancel = OKCancel()
-        okcancel.ok.setText(translate('Edit Field', 'A&dd'))
+        okcancel.okButton.setText(translate('Edit Field', 'A&dd'))
 
         if field is not None:
             x = self.tagcombo.findText(field[0])
@@ -415,34 +418,35 @@ class EditField(QDialog):
                 self.tagcombo.setEditText(field[0])
             self.value.setPlainText(field[1])
             if edit:
-                okcancel.ok.setText(translate('Edit Field', 'E&dit'))
+                okcancel.okButton.setText(translate('Edit Field', 'E&dit'))
 
-        map(self.vbox.addWidget, [label, self.tagcombo, label1, self.value])
+        list(map(self.vbox.addWidget, [label, self.tagcombo, label1, self.value]))
 
         self.vbox.addLayout(okcancel)
         self.setLayout(self.vbox)
 
-        self.connect(okcancel, SIGNAL("ok"), self.ok)
-        self.connect(okcancel, SIGNAL("cancel"), self.close)
+        okcancel.ok.connect(self.ok)
+        okcancel.cancel.connect(self.close)
 
         self.value.setFocus() if self.__oldField else self.tagcombo.setFocus()
 
     def ok(self):
         self.close()
-        self.emit(SIGNAL("donewithmyshit"),
-            unicode(self.tagcombo.currentText()),
-            unicode(self.value.toPlainText()),
+        self.donewithmyshit.emit(
+            str(self.tagcombo.currentText()),
+            str(self.value.toPlainText()),
             self.__oldField)
 
+
 class StatusWidgetItem(QTableWidgetItem):
     def __init__(self, text=None, status=None, colors=None, preview=False):
         QTableWidgetItem.__init__(self)
         self.preview = preview
         self.statusColors = colors
-        
+
         if text:
             self.setText(text)
-            
+
         if status and status in self.statusColors:
             self.setBackground(self.statusColors[status])
 
@@ -477,7 +481,7 @@ class StatusWidgetItem(QTableWidgetItem):
     status = property(_get_status, _set_status)
 
     def __lt__(self, item):
-        if self.text().toUpper() < item.text().toUpper():
+        if self.text().upper() < item.text().upper():
             return True
         return False
 
@@ -493,17 +497,18 @@ class StatusWidgetItem(QTableWidgetItem):
 
 
 class VerticalHeader(QHeaderView):
-    def __init__(self, parent = None):
+    def __init__(self, parent=None):
         QHeaderView.__init__(self, Qt.Vertical, parent)
         self.setDefaultSectionSize(self.minimumSectionSize() + 4)
         self.setMinimumSectionSize(1)
 
-        self.connect(self, SIGNAL('sectionResized(int, int, int)'),
+        self.sectionResized.connect(
             self._resize)
 
     def _resize(self, row, oldsize, newsize):
         self.setDefaultSectionSize(newsize)
 
+
 class StatusWidgetCombo(QComboBox):
     def __init__(self, items=None, status=None, colors=None, preview=False):
         QComboBox.__init__(self)
@@ -511,20 +516,21 @@ class StatusWidgetCombo(QComboBox):
 
         self.statusColors = colors
         self.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLength)
-        
-        items = sorted(items, cmp=natcasecmp)
+
+        items = map(str, items)
+        items = sorted(items, key=natsort_case_key)
         if len(items) > 1:
             items.append(r'\\'.join(items))
         self.addItem(KEEP)
         self.addItems(items)
         self.setCurrentIndex(0)
-            
+
         if status and status in self.statusColors:
             self.setBackground(self.statusColors[status])
         else:
             self.setBackground(None)
 
-        self._status = status 
+        self._status = status
         self._original = (items, preview, status)
         self.linked = []
 
@@ -555,7 +561,7 @@ class StatusWidgetCombo(QComboBox):
 
     status = property(_get_status, _set_status)
 
-    def setBackground(self, brush = None):
+    def setBackground(self, brush=None):
         if brush is None:
             color = QLineEdit().palette().color(QPalette.Base).name()
         else:
@@ -586,18 +592,22 @@ class StatusWidgetCombo(QComboBox):
         else:
             self.setItemText(self.currentIndex(), value)
 
+
 class ExTags(QDialog):
     """A dialog that shows you the tags in a file
 
     In addition, any attached cover art is shown."""
+    rowChanged = pyqtSignal(object, name='rowChanged')
+    extendedtags = pyqtSignal(dict, name='extendedtags')
+
     def __init__(self, parent=None, row=None, files=None, preview_mode=False,
-        artwork=True, status=None):
+                 artwork=True, status=None):
 
         if status is None:
             status = {'cover_pattern': 'folder'}
 
         self.status = status
-            
+
         QDialog.__init__(self, parent)
         winsettings('extendedtags', self)
         self.get_fieldlist = []
@@ -607,7 +617,7 @@ class ExTags(QDialog):
         edit = QColor.fromRgb(0, 255, 0)
         remove = QColor.fromRgb(255, 0, 0)
         self._colors = {ADD: QBrush(add),
-            EDIT: QBrush(edit), REMOVE: QBrush(remove)}
+                        EDIT: QBrush(edit), REMOVE: QBrush(remove)}
 
         self.table = QTableWidget(0, 2, self)
         self.table.setVerticalHeader(VerticalHeader())
@@ -620,24 +630,23 @@ class ExTags(QDialog):
 
         header = self.table.horizontalHeader()
         header.setVisible(True)
-        header.setSortIndicatorShown (True)
-        header.setStretchLastSection (True)
-        header.setSortIndicator (0, Qt.AscendingOrder)
+        header.setSortIndicatorShown(True)
+        header.setStretchLastSection(True)
+        header.setSortIndicator(0, Qt.AscendingOrder)
 
-        self.piclabel = PicWidget(buttons = True)
-        self.connect(self.piclabel, SIGNAL('imageChanged'),
+        self.piclabel = PicWidget(buttons=True)
+        self.piclabel.imageChanged.connect(
             self._imageChanged)
 
         if not isinstance(self.piclabel.removepic, QAction):
-            self.connect(self.piclabel.removepic, SIGNAL('clicked()'),
+            self.piclabel.removepic.clicked.connect(
                 self.removePic)
         else:
-            self.connect(self.piclabel.removepic,
-                SIGNAL('triggered()'), self.removePic)
+            self.piclabel.removepic.triggered.connect(self.removePic)
 
-        if row >= 0 and files:
+        if row and row >= 0 and files:
             buttons = MoveButtons(files, row)
-            self.connect(buttons, SIGNAL('indexChanged'), self._prevnext)
+            buttons.indexChanged.connect(self._prevnext)
             buttons.setVisible(True)
         else:
             buttons = MoveButtons([], row)
@@ -650,14 +659,14 @@ class ExTags(QDialog):
 
         self._reset = QToolButton()
         self._reset.setToolTip(translate('Extended Tags',
-                'Resets the selected fields to their original value.'))
+                                         'Resets the selected fields to their original value.'))
         self._reset.setIcon(get_icon('edit-undo', ':/undo.png'))
-        self.connect(self._reset, SIGNAL('clicked()'), self.resetFields)
+        self._reset.clicked.connect(self.resetFields)
 
         self.listbuttons = ListButtons()
         self.listbuttons.layout().addWidget(self._reset)
-        self.listbuttons.moveup.hide()
-        self.listbuttons.movedown.hide()
+        self.listbuttons.moveupButton.hide()
+        self.listbuttons.movedownButton.hide()
 
         listframe = QFrame()
         listframe.setFrameStyle(QFrame.Box)
@@ -671,7 +680,7 @@ class ExTags(QDialog):
             imageframe = QFrame()
             imageframe.setFrameStyle(QFrame.Box)
             vbox = QVBoxLayout()
-            vbox.setMargin(0)
+            vbox.setContentsMargins(0, 0, 0, 0)
             vbox.addWidget(self.piclabel)
             vbox.addStretch()
             vbox.addStrut(0)
@@ -689,25 +698,22 @@ class ExTags(QDialog):
         layout.addLayout(self.okcancel)
         self.setLayout(layout)
 
-        self.connect(self.okcancel, SIGNAL("cancel"), self.closeMe)
-        self.connect(self.table,
-            SIGNAL("itemDoubleClicked(QTableWidgetItem *)"), self.editField)
-        self.connect(self.table,
-            SIGNAL("itemSelectionChanged()"), self._checkListBox)
-        self.connect(self.okcancel, SIGNAL("ok"), self.okClicked)
-        
-        clicked = SIGNAL('clicked()')
-        self.connect(self.listbuttons, SIGNAL('edit'), self.editField)
-        self.connect(self.listbuttons.add, clicked, self.addField)
-        self.connect(self.listbuttons.remove, clicked, self.removeField)
-        self.connect(self.listbuttons, SIGNAL('duplicate'), self.duplicate)
+        self.okcancel.cancel.connect(self.closeMe)
+        self.table.itemDoubleClicked.connect(self.editField)
+        self.table.itemSelectionChanged.connect(self._checkListBox)
+        self.okcancel.ok.connect(self.okClicked)
+
+        self.listbuttons.edit.connect(self.editField)
+        self.listbuttons.addButton.clicked.connect(self.addField)
+        self.listbuttons.removeButton.clicked.connect(self.removeField)
+        self.listbuttons.duplicate.connect(self.duplicate)
 
         self.setMinimumSize(450, 350)
 
         self.canceled = False
         self.filechanged = False
 
-        if row >= 0 and files:
+        if row and row >= 0 and files:
             self._prevnext(row)
         else:
             self.loadFiles(files)
@@ -716,30 +722,30 @@ class ExTags(QDialog):
         win = EditField(parent=self, field_list=self.get_fieldlist)
         win.setModal(True)
         win.show()
-        self.connect(win, SIGNAL("donewithmyshit"), self.editFieldBuddy)
+        win.donewithmyshit.connect(self.editFieldBuddy)
 
     def _checkListBox(self):
         if self.table.rowCount() <= 0:
             self.table.setEnabled(False)
-            self.listbuttons.edit.setEnabled(False)
-            self.listbuttons.remove.setEnabled(False)
-            self.listbuttons.duplicate.setEnabled(False)
+            self.listbuttons.editButton.setEnabled(False)
+            self.listbuttons.removeButton.setEnabled(False)
+            self.listbuttons.duplicateButton.setEnabled(False)
             self._reset.setEnabled(False)
         else:
             self.table.setEnabled(True)
             self._reset.setEnabled(True)
             if len(self.table.selectedIndexes()) / 2 > 1:
-                self.listbuttons.edit.setEnabled(False)
-                self.listbuttons.duplicate.setEnabled(False)
+                self.listbuttons.editButton.setEnabled(False)
+                self.listbuttons.duplicateButton.setEnabled(False)
             else:
-                self.listbuttons.edit.setEnabled(True)
-                self.listbuttons.remove.setEnabled(True)
-                self.listbuttons.duplicate.setEnabled(True)
+                self.listbuttons.editButton.setEnabled(True)
+                self.listbuttons.removeButton.setEnabled(True)
+                self.listbuttons.duplicateButton.setEnabled(True)
         self.table.resizeColumnToContents(0)
-    
-    def closeEvent(self,event):
+
+    def closeEvent(self, event):
         self.piclabel.close()
-        QDialog.closeEvent(self,event)
+        QDialog.closeEvent(self, event)
 
     def closeMe(self):
         self.canceled = True
@@ -769,37 +775,37 @@ class ExTags(QDialog):
             win.setModal(True)
             win.show()
 
-            #Have to check for truth, because this method is
-            #called by the doubleclicked signal.
+            # Have to check for truth, because this method is
+            # called by the doubleclicked signal.
             if duplicate is True:
                 buddy = partial(self.editFieldBuddy, duplicate=True)
             else:
                 buddy = self.editFieldBuddy
-            self.connect(win, SIGNAL("donewithmyshit"), buddy)
+            win.donewithmyshit.connect(buddy)
 
-    def editFieldBuddy(self, tag, value, prevtag = None, duplicate=False):
+    def editFieldBuddy(self, tag, value, prevtag=None, duplicate=False):
         rowcount = self.table.rowCount()
         if prevtag is not None:
             if duplicate:
                 row = rowcount
                 self._settag(rowcount, tag, value, ADD,
-                    self.previewMode, True)
+                             self.previewMode, True)
             else:
                 if tag == prevtag[0]:
                     row = self.table.currentRow()
                     self._settag(row, tag, value, EDIT,
-                        self.previewMode, True)
+                                 self.previewMode, True)
                     if row + 1 < rowcount:
                         self.table.selectRow(row + 1)
                 else:
-                    cur_item = self.table.item(self.table.currentRow(),0)
+                    cur_item = self.table.item(self.table.currentRow(), 0)
                     self.resetFields([cur_item])
                     self.table.setCurrentItem(cur_item,
-                        QItemSelectionModel.ClearAndSelect)
+                                              QItemSelectionModel.ClearAndSelect)
                     self.table.selectRow(self.table.row(cur_item))
                     self.removeField()
                     valitem = self._settag(rowcount, tag,
-                        value, ADD, self.previewMode, True)
+                                           value, ADD, self.previewMode, True)
                     cur_item.linked = [valitem]
         else:
             self._settag(rowcount, tag, value, ADD, self.previewMode, True)
@@ -807,14 +813,14 @@ class ExTags(QDialog):
         self.filechanged = True
         self.table.clearSelection()
 
-    def get_field(self, row, status = None):
+    def get_field(self, row, status=None):
         getitem = self.table.item
         item = getitem(row, 0)
-        tag = unicode(item.text())
+        tag = str(item.text())
         try:
-            value = unicode(getitem(row, 1).text())
+            value = str(getitem(row, 1).text())
         except AttributeError:
-            value = unicode(self.table.cellWidget(row, 1).currentText())
+            value = str(self.table.cellWidget(row, 1).currentText())
         if status:
             return (tag, value, item.status)
         else:
@@ -826,20 +832,20 @@ class ExTags(QDialog):
     def loadSettings(self):
         cparser = PuddleConfig()
         self.get_fieldlist = gettaglist()
-        get = lambda k,v : cparser.get('extendedtags', k, v, True)
+        get = lambda k, v: cparser.get('extendedtags', k, v, True)
         add = QColor.fromRgb(*get('add', [255, 255, 0]))
-        edit = QColor.fromRgb(*get('edit', [0, 255,0]))
+        edit = QColor.fromRgb(*get('edit', [0, 255, 0]))
         remove = QColor.fromRgb(*get('remove', [255, 0, 0]))
 
         self._colors = {ADD: QBrush(add),
-            EDIT: QBrush(edit), REMOVE: QBrush(remove)}
+                        EDIT: QBrush(edit), REMOVE: QBrush(remove)}
 
         item = self.table.item
-        for row in xrange(self.table.rowCount()):
+        for row in range(self.table.rowCount()):
             field_item = self.get_item(row, 0)
             field_item.statusColors = self._colors
             field_item.status = field_item.status
-            
+
             val_item = self.get_item(row, 1)
             val_item.statusColors = self._colors
             val_item.status = val_item.status
@@ -849,7 +855,7 @@ class ExTags(QDialog):
         tags = {}
         lowered = {}
         listitems = [get_field(row, True) for row
-            in xrange(self.table.rowCount())]
+                     in range(self.table.rowCount())]
 
         for field, val, status in listitems:
             if status != REMOVE:
@@ -884,12 +890,12 @@ class ExTags(QDialog):
         else:
             self.setWindowTitle(
                 translate('Extended Tags', 'Different files.'))
-            
-            from puddlestuff.tagmodel import status
+
+            from .tagmodel import status
             k = status['table'].model().taginfo[0]
             common, numvalues, imagetags = commontags(audios)
             images = common['__image']
-            del(common['__image'])
+            del (common['__image'])
             previews = set(audios[0].preview)
             italics = set(audios[0].equal_fields())
             self.piclabel.currentFile = audios[0]
@@ -901,10 +907,10 @@ class ExTags(QDialog):
 
             row = 0
 
-            for field, values in common.iteritems():
+            for field, values in common.items():
                 if field in italics:
                     preview = UNCHANGED
-                #field in italics => field in previews.
+                # field in italics => field in previews.
                 elif field in previews:
                     preview = BOLD
                 else:
@@ -913,7 +919,7 @@ class ExTags(QDialog):
                     self._settag(row, field, values, multi=True)
                     row += 1
                 else:
-                    if isinstance(values, basestring):
+                    if isinstance(values, str):
                         self._settag(row, field, values, None, preview)
                         row += 1
                     else:
@@ -947,7 +953,7 @@ class ExTags(QDialog):
                 preview = BOLD
             else:
                 preview = UNCHANGED
-            if isinstance(val, basestring):
+            if isinstance(val, str):
                 items.append([key, val, None, preview])
             else:
                 [items.append([key, z, None, preview]) for z in val]
@@ -978,10 +984,10 @@ class ExTags(QDialog):
         if self.filechanged:
             self.save()
         self.loadFiles([self._files[row]])
-        self.emit(SIGNAL('rowChanged'), row)
+        self.rowChanged.emit(row)
 
     def get_item(self, row, column=None):
-        if column is None: #Assume QModelIndex passed
+        if column is None:  # Assume QModelIndex passed
             column = row.column()
             row = row.row()
         item = self.table.item(row, column)
@@ -1014,14 +1020,14 @@ class ExTags(QDialog):
 
     def resetFields(self, items=None):
         box = self.table
-        to_remove = {} #Stores row: item values so that only one item
-                       #gets removed per row.
+        to_remove = {}  # Stores row: item values so that only one item
+        # gets removed per row.
 
         max_row = -1
         for index in box.selectedIndexes():
             row = index.row()
             item = self.table.item(row, index.column())
-            if item is None:    
+            if item is None:
                 item = self.table.cellWidget(row, index.column())
             for i in item.linked:
                 try:
@@ -1047,17 +1053,15 @@ class ExTags(QDialog):
             self.piclabel.context = 'No Images'
             self.piclabel.removepic.setEnabled(False)
             if not isinstance(self.piclabel.removepic, QAction):
-                self.disconnect(self.piclabel.removepic, SIGNAL('clicked()'),
+                self.piclabel.removepic.clicked.disconnect(
                     self.removePic)
             else:
-                self.discconnect(self.piclabel.removepic,
-                    SIGNAL('triggered()'), self.removePic)
-            
-            
+                self.piclabel.removepic.triggered.disconnect(self.removePic)
+
             self.piclabel.setImages(None)
 
     def save(self):
-        
+
         if not self.filechanged:
             table = self.table
             for row in range(table.rowCount()):
@@ -1078,7 +1082,7 @@ class ExTags(QDialog):
         newtags = [z for z in tags if z not in self.get_fieldlist]
         if newtags and newtags != ['__image']:
             settaglist(newtags + self.get_fieldlist)
-        self.emit(SIGNAL('extendedtags'), tags)
+        self.extendedtags.emit(tags)
 
     def _settag(self, row, field, value, status=None, preview=False,
                 check=False, multi=False):
@@ -1088,9 +1092,9 @@ class ExTags(QDialog):
         if row >= tb.rowCount():
             tb.insertRow(row)
             field_item = StatusWidgetItem(field, status,
-                self._colors, preview)
+                                          self._colors, preview)
             tb.setItem(row, 0, field_item)
-            if not multi and (len(value) == 1 or isinstance(value, basestring)):
+            if not multi and (len(value) == 1 or isinstance(value, str)):
                 valitem = StatusWidgetItem(to_string(value), status, self._colors, preview)
                 tb.setItem(row, 1, valitem)
             else:
@@ -1100,16 +1104,16 @@ class ExTags(QDialog):
             field_item = tb.item(row, 0)
             field_item.setText(field)
             field_item.status = status
-            
+
             val_item = self.get_item(row, 1)
             val_item.setText(value)
             val_item.status = status
 
         if check:
             lowered_tag = field.lower()
-            for row in xrange(tb.rowCount()):
+            for row in range(tb.rowCount()):
                 item = tb.item(row, 0)
-                text = unicode(item.text())
+                text = str(item.text())
                 if text != field and text.lower() == lowered_tag:
                     item.setText(field)
                     if item.status not in [ADD, REMOVE]:
@@ -1122,6 +1126,7 @@ class ExTags(QDialog):
         tb.setSortingEnabled(True)
         return field_item
 
+
 class ConfirmationErrorDialog(QDialog):
     def __init__(self, name, parent=None):
         super(ConfirmationErrorDialog, self).__init__(parent)
@@ -1135,7 +1140,7 @@ class ConfirmationErrorDialog(QDialog):
         checkbox = QCheckBox(
             translate("Defaults", "Never show this message again."))
 
-        self.connect(ok, SIGNAL('clicked()'), partial(self.saveState, name))
+        ok.clicked.connect(partial(self.saveState, name))
 
         labelbox = QHBoxLayout()
         labelbox.addWidget(icon)
@@ -1160,13 +1165,14 @@ class ConfirmationErrorDialog(QDialog):
         if should_show:
             self.setModal(True)
             self.show()
-            
+
 
 if __name__ == "__main__":
     app = QApplication(sys.argv)
     dirpath = '/home/keith/Desktop/Daft.Punk-Random.Access.Memories-2013-FLAC.-NewsHost-1023'
     import glob
-    tags = map(puddlestuff.audioinfo.Tag, glob.glob(os.path.join(dirpath, "*.flac")))
+
+    tags = list(map(audioinfo.Tag, glob.glob(os.path.join(dirpath, "*.flac"))))
     for tag in tags:
         tag.preview = {}
         tag.equal_fields = lambda: []
