@@ -1,24 +1,21 @@
-# -*- coding: utf-8 -*-
-import pdb, string
-
 from collections import defaultdict
 from copy import deepcopy
 from operator import itemgetter
 
-import puddlestuff
+from ..audioinfo import FILENAME
+from ..constants import VARIOUS
+from ..findfunc import filenametotag
+from ..puddleobjects import natsort_case_key, ratio
+from ..tagsources import RetrievalError
+from ..translations import translate
+from ..util import sorted_split_by_field, split_by_field, to_string
+from ..webdb import (strip as strip_fields, DEFAULT_REGEXP,
+                     apply_regexps)
 
-from puddlestuff.audioinfo import FILENAME
-from puddlestuff.constants import VARIOUS
-from puddlestuff.findfunc import filenametotag
-from puddlestuff.puddleobjects import natcasecmp, ratio
-from puddlestuff.tagsources import RetrievalError
-from puddlestuff.translations import translate
-from puddlestuff.util import sorted_split_by_field, split_by_field, to_string
-from puddlestuff.webdb import (strip as strip_fields, DEFAULT_REGEXP,
-    apply_regexps)
 
 def set_status(v):
-    print v
+    print(v)
+
 
 NO_MATCH_OPTIONS = [
     translate('Masstagging', 'Continue'),
@@ -56,53 +53,54 @@ NAME = 'name'
 DESC = 'description'
 EXISTING_ONLY = 'field_exists'
 
-DEFAULT_PATTERN = u'%artist% - %album%/%track% - %title%'
+DEFAULT_PATTERN = '%artist% - %album%/%track% - %title%'
 DEFAULT_NAME = translate('Masstagging', 'Default Profile')
 
 POLLING = translate("Masstagging", '<b>Polling: %s</b>')
 MATCH_ARTIST_ALBUM = translate("Masstagging",
-    'Retrieving matching album. <b>%1 - %2</b>')
+                               'Retrieving matching album. <b>%1 - %2</b>')
 MATCH_ARTIST = translate("Masstagging",
-    'Retrieving matching album. Artist=<b>%1</b>')
+                         'Retrieving matching album. Artist=<b>%1</b>')
 MATCH_ALBUM = translate("Masstagging",
-    'Retrieving matching album. Album=<b>%1</b>')
+                        'Retrieving matching album. Album=<b>%1</b>')
 MATCH_NO_INFO = translate("Masstagging", 'Retrieving matching album.')
 
-SEARCHING_ARTIST_ALBUM = u':insert' + translate("Masstagging",
-    'Starting search for: <br />artist=<b>%1</b> '
-    '<br />album=<b>%2</b><br />')
-SEARCHING_ARTIST = u':insert' + translate("Masstagging",
-    'Starting search for: <br />artist=<b>%1</b>'
-    '<br />album=No album name found.')
-SEARCHING_ALBUM = u':insert' + translate("Masstagging",
-    'Starting search for: <br />album=<b>%1</b>'
-    '<br />artist=No artist found.')
-SEARCHING_NO_INFO = u':insert' + translate("Masstagging",
-    'No artist or album info found in files. Starting search.')
+SEARCHING_ARTIST_ALBUM = ':insert' + translate("Masstagging",
+                                               'Starting search for: <br />artist=<b>%1</b> '
+                                               '<br />album=<b>%2</b><br />')
+SEARCHING_ARTIST = ':insert' + translate("Masstagging",
+                                         'Starting search for: <br />artist=<b>%1</b>'
+                                         '<br />album=No album name found.')
+SEARCHING_ALBUM = ':insert' + translate("Masstagging",
+                                        'Starting search for: <br />album=<b>%1</b>'
+                                        '<br />artist=No artist found.')
+SEARCHING_NO_INFO = ':insert' + translate("Masstagging",
+                                          'No artist or album info found in files. Starting search.')
 
 RESULTS_FOUND = translate("Masstagging", '<b>%d</b> results found.')
 NO_RESULTS_FOUND = translate("Masstagging", '<b>No results were found.</b>')
 ONE_RESULT_FOUND = translate("Masstagging", '<b>One</b> result found.')
 
 MATCHING_ALBUMS_FOUND = translate("Masstagging",
-    '<b>%d</b> possibly matching albums found.')
+                                  '<b>%d</b> possibly matching albums found.')
 ONE_MATCHING_ALBUM_FOUND = translate("Masstagging",
-    '<b>One</b> possibly matching album found.')
+                                     '<b>One</b> possibly matching album found.')
 NO_MATCHES = translate("Masstagging",
-    'No matches found for tag source <b>%s</b>')
+                       'No matches found for tag source <b>%s</b>')
 
 RETRIEVING_NEXT = translate("Masstagging",
-    'Previously retrieved result does not match. '
-    'Retrieving next matching album.')
+                            'Previously retrieved result does not match. '
+                            'Retrieving next matching album.')
 
 RECHECKING = translate("Masstagging",
-    '<br />Rechecking with results from <b>%s</b>.<br />')
+                       '<br />Rechecking with results from <b>%s</b>.<br />')
 
 VALID_FOUND = translate("Masstagging",
-    '<br />Valid matches were found for the album.')
+                        '<br />Valid matches were found for the album.')
 
 NO_VALID_FOUND = translate("Masstagging",
-    '<b>No valid matches were found for the album.</b>')
+                           '<b>No valid matches were found for the album.</b>')
+
 
 class MassTagFlag(object):
     def __init__(self):
@@ -110,24 +108,23 @@ class MassTagFlag(object):
         object.__init__(self)
 
 
-
 def brute_force_results(audios, retrieved):
     matched = {}
 
-    audios = sorted(audios, natcasecmp,
-        lambda f: to_string(f.get('track', f['__filename'])))
+    audios = sorted(audios, natsort_case_key,
+                    lambda f: to_string(f.get('track', f['__filename'])))
 
-    retrieved = sorted(retrieved, natcasecmp,
-        lambda t: to_string(t.get('track', t.get('title' , u''))))
+    retrieved = sorted(retrieved, natsort_case_key,
+                       lambda t: to_string(t.get('track', t.get('title', ''))))
 
     for audio, result in zip(audios, retrieved):
         matched[audio] = result
 
     return matched
 
+
 def check_result(result, audios):
-    track_nums = filter(None,
-        [to_string(audio.get('track', None)) for audio in audios])
+    track_nums = [_f for _f in [to_string(audio.get('track', None)) for audio in audios] if _f]
 
     if result.tracks is None:
         return True
@@ -135,8 +132,10 @@ def check_result(result, audios):
     if track_nums:
         max_num = 0
         for num in track_nums:
-            try: num = int(num)
-            except (TypeError, ValueError): continue
+            try:
+                num = int(num)
+            except (TypeError, ValueError):
+                continue
             max_num = num if num > max_num else max_num
         if max_num != 0 and max_num == len(result.tracks):
             return True
@@ -145,11 +144,12 @@ def check_result(result, audios):
         return True
     return False
 
+
 def combine_tracks(track1, track2, repl=None):
     ret = defaultdict(lambda: [])
-    
-    for key, value in track2.items() + track1.items():
-        if isinstance(value, basestring):
+
+    for key, value in list(track2.items()) + list(track1.items()):
+        if isinstance(value, str):
             if value not in ret[key]:
                 ret[key].append(value)
         else:
@@ -166,10 +166,12 @@ def combine_tracks(track1, track2, repl=None):
         ret['#exact'] = track2['#exact']
     return ret
 
+
 def fields_from_text(text):
     if not text:
         return []
-    return filter(None, map(string.strip, text.split(u',')))
+    return [_f for _f in map(str.strip, text.split(',')) if _f]
+
 
 def dict_difference(dict1, dict2):
     """Returns a dictonary containing key/value pairs from dict2 where key
@@ -180,10 +182,11 @@ def dict_difference(dict1, dict2):
             temp[field] = dict2[field]
     return temp
 
+
 def find_best(matches, files, minimum=0.7):
     group = split_by_field(files, 'album', 'artist')
-    album = group.keys()[0]
-    artists = group[album].keys()
+    album = list(group.keys())[0]
+    artists = list(group[album].keys())
     if len(artists) == 1:
         artist = artists[0]
     else:
@@ -200,7 +203,7 @@ def find_best(matches, files, minimum=0.7):
         score = min([ratio_compare(d, info, key) for key in d])
 
         if score in scores:
-            score = score + 0.01 #For albums that have same name.
+            score = score + 0.01  # For albums that have same name.
         scores[score] = match
         tracks = match.tracks if hasattr(match, 'tracks') else match[1]
         if tracks and score < minimum:
@@ -209,19 +212,21 @@ def find_best(matches, files, minimum=0.7):
 
     if scores:
         return [scores[z] for z in
-            sorted(scores, reverse=True) if z >= minimum]
+                sorted(scores, reverse=True) if z >= minimum]
     else:
         return []
 
+
 def get_artist_album(files):
     tags = split_by_field(files, 'album', 'artist')
-    album = tags.keys()[0]
+    album = list(tags.keys())[0]
     artists = tags[album]
     if len(artists) > 1:
         return VARIOUS, album
     else:
         return list(artists)[0], album
 
+
 def get_match_str(info):
     artist = album = None
     if info.get('artist'):
@@ -239,14 +244,16 @@ def get_match_str(info):
     else:
         return MATCH_NO_INFO
 
-get_lower = lambda f, key, default=u'': to_string(f.get(key,default)).lower()
+
+get_lower = lambda f, key, default='': to_string(f.get(key, default)).lower()
+
 
 def ratio_compare(d1, d2, key):
-    return ratio(get_lower(d1, key, u'a'), get_lower(d2, key, u'b'))
+    return ratio(get_lower(d1, key, 'a'), get_lower(d2, key, 'b'))
+
 
 def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
-    existing=False, as_index=False):
-
+                existing=False, as_index=False):
     if not keys:
         keys = ['artist', 'title']
     if 'track' in keys and len(keys) > 1:
@@ -257,7 +264,7 @@ def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
     assigned = {}
     matched = defaultdict(lambda: {})
     b = False
-    
+
     for f_index, f in enumerate(files):
         scores = {}
         for t_index, track in enumerate(tracks):
@@ -271,7 +278,7 @@ def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
     def get_best(f_index, t_indexes):
         if not t_indexes:
             return
-        items = t_indexes.items()
+        items = list(t_indexes.items())
         best_match = max(items, key=itemgetter(1))
         t_i = best_match[0]
 
@@ -294,8 +301,8 @@ def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
 
         assigned[t_i] = f_index
 
-    for f_index, t_indexes in matched.iteritems():
-        best_match = max(t_indexes.items(), key=itemgetter(1))
+    for f_index, t_indexes in matched.items():
+        best_match = max(list(t_indexes.items()), key=itemgetter(1))
         t_index = best_match[0]
         while t_index in assigned:
             prev_matched = assigned[t_index]
@@ -303,17 +310,17 @@ def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
                 get_best(prev_matched, matched[prev_matched])
                 break
             else:
-                del(t_indexes[t_index])
+                del (t_indexes[t_index])
                 if not t_indexes:
                     break
-                best_match = max(t_indexes.items(), key=itemgetter(1))
+                best_match = max(list(t_indexes.items()), key=itemgetter(1))
                 t_index = best_match[0]
 
         if t_indexes:
             assigned[t_index] = f_index
 
     ret_indexes = {}
-    for t_index, f_index in assigned.iteritems():
+    for t_index, f_index in assigned.items():
         try:
             ret[files[f_index].cls] = tracks[t_index]
             ret_indexes[t_index] = files[f_index].cls
@@ -324,34 +331,38 @@ def match_files(files, tracks, minimum=0.7, keys=None, jfdi=False,
     for t in tracks:
         if '#exact' in t:
             ret[t['#exact'].cls] = t
-            del(t['#exact'])
+            del (t['#exact'])
 
     if jfdi:
         unmatched_tracks = [t for i, t in enumerate(tracks) if i
-            not in assigned]
+                            not in assigned]
         unmatched_files = [f.cls for f in files if f.cls not in ret]
         ret.update(brute_force_results(unmatched_files, unmatched_tracks))
 
     if existing:
-        ret = dict((f, dict_difference(f, r)) for f, r in ret.iteritems())
+        ret = dict((f, dict_difference(f, r)) for f, r in ret.items())
 
     if as_index:
         return ret, ret_indexes
     return ret
 
+
 def merge_track(audio, info):
     track = {}
 
     for key in info.keys():
         if not key.startswith('#'):
-            if isinstance(info[key], basestring):
+            if isinstance(info[key], str):
                 track[key] = info[key]
             else:
-                track[key] = info[key][::]
+                if isinstance(info[key], list):
+                    track[key] = info[key][::]
+                elif isinstance(info[key], dict):
+                    track[key] = info[key]
 
     for key in audio.keys():
         if not key.startswith('#'):
-            if isinstance(audio[key], basestring):
+            if isinstance(audio[key], str):
                 track[key] = audio[key]
             else:
                 track[key] = audio[key][::]
@@ -372,7 +383,7 @@ def merge_tsp_tracks(profiles, files=None):
             tags = [deepcopy(info) for z in files]
         else:
             tags = [strip_fields(t, tsp.fields, leave_exact=True)
-                for t in tsp.result.merged]
+                    for t in tsp.result.merged]
 
         if len(tags) > len(ret):
             ret.extend(tags[len(ret):])
@@ -387,9 +398,9 @@ def merge_tsp_tracks(profiles, files=None):
 
     return ret
 
-def masstag(mtp, files=None, flag=None, mtp_error_func=None,
-    tsp_error_func=None, print_status=True):
 
+def masstag(mtp, files=None, flag=None, mtp_error_func=None,
+            tsp_error_func=None, print_status=True):
     not_found = []
     found = []
 
@@ -439,7 +450,7 @@ def masstag(mtp, files=None, flag=None, mtp_error_func=None,
         result = tsp.retrieve(matches[0], errors=tsp_error_func)
 
         while not check_result(result, files):
-            del(matches[0])
+            del (matches[0])
             if matches:
                 set_status(RETRIEVING_NEXT)
                 set_status(get_match_str(matches[0].info))
@@ -447,7 +458,7 @@ def masstag(mtp, files=None, flag=None, mtp_error_func=None,
             else:
                 result = None
                 break
-            
+
         if result is None:
             set_status(NO_MATCHES % tsp.tag_source.name)
             not_found.append(tsp)
@@ -459,16 +470,16 @@ def masstag(mtp, files=None, flag=None, mtp_error_func=None,
     if not_found and found:
         set_status(RECHECKING % found[0].tag_source.name)
         audios_copy = []
-        for t, m in zip(map(deepcopy, files), found[0].result.merged):
-            audios_copy.append(combine_tracks(t,m))
+        for t, m in zip(list(map(deepcopy, files)), found[0].result.merged):
+            audios_copy.append(combine_tracks(t, m))
 
         new_mtp = MassTagProfile(translate("Masstagging", 'Rechecking'),
-            files=audios_copy, profiles=not_found,
-            album_bound=mtp.album_bound, track_bound=mtp.track_bound,
-            regexps=mtp.regexps)
+                                 files=audios_copy, profiles=not_found,
+                                 album_bound=mtp.album_bound, track_bound=mtp.track_bound,
+                                 regexps=mtp.regexps)
 
         ret = masstag(new_mtp, audios_copy, flag,
-            mtp_error_func, tsp_error_func, False)
+                      mtp_error_func, tsp_error_func, False)
 
     if found:
         if not ret and print_status:
@@ -479,15 +490,15 @@ def masstag(mtp, files=None, flag=None, mtp_error_func=None,
             set_status(NO_VALID_FOUND)
         return []
 
+
 def replace_tracknumbers(files, tracks):
     if len(files) != len(tracks):
         return
 
-    files = sorted(files, cmp=natcasecmp,
-        key=lambda f: to_string(f.get('track', f[FILENAME])))
+    files = sorted(files, key=lambda f: to_string(f.get('track', f[FILENAME])))
     try:
-        tracks = sorted(tracks, cmp=natcasecmp, key=itemgetter('track'))
-        tracks = sorted(tracks, cmp=natcasecmp, key=itemgetter('discnumber'))
+        tracks = sorted(tracks, key=itemgetter('track'))
+        tracks = sorted(tracks, key=itemgetter('discnumber'))
     except KeyError:
         return
 
@@ -510,10 +521,10 @@ def replace_tracknumbers(files, tracks):
             except (ValueError, TypeError, KeyError):
                 continue
             if f_tracknum > t_tracknum:
-                f['track'] = [unicode(f_tracknum - offset )]
+                f['track'] = [str(f_tracknum - offset)]
+
 
 def split_files(audios, pattern):
-
     def copy_audio(f):
         tags = filenametotag(pattern, f['__path'], True)
         audio_copy = deepcopy(f)
@@ -526,23 +537,25 @@ def split_files(audios, pattern):
     for dirpath, files in sorted_split_by_field(audios, '__dirpath'):
         album_groups = sorted_split_by_field(files, 'album')
         for album, album_files in album_groups:
-            tag_groups.append(map(copy_audio, album_files))
+            tag_groups.append(list(map(copy_audio, album_files)))
 
     return tag_groups
 
+
 def to_int(v):
     return int(to_string(v))
 
+
 class MassTagProfile(object):
-    def __init__(self, name=DEFAULT_NAME, desc=u'', fields=None, files=None,
-        file_pattern=DEFAULT_PATTERN, profiles=None, album_bound=0.50,
-        track_bound=0.80, jfdi=True, leave_existing=False, regexps=''):
+    def __init__(self, name=DEFAULT_NAME, desc='', fields=None, files=None,
+                 file_pattern=DEFAULT_PATTERN, profiles=None, album_bound=0.50,
+                 track_bound=0.80, jfdi=True, leave_existing=False, regexps=''):
 
         object.__init__(self)
 
         self.album_bound = album_bound
         self.desc = desc
-        self.fields = [u'artist', u'title'] if fields is None else fields
+        self.fields = ['artist', 'title'] if fields is None else fields
         self.file_pattern = file_pattern
         self.files = [] if files is None else files
         self.jfdi = jfdi
@@ -568,7 +581,7 @@ class MassTagProfile(object):
 
         if regexps:
             changed_files = \
-                map(lambda f: apply_regexps(f, regexps), files)
+                [apply_regexps(f, regexps) for f in files]
             rxp_album = changed_files[0][0]
             changed_files = [z[1] for z in changed_files]
 
@@ -580,13 +593,13 @@ class MassTagProfile(object):
                 if regexps and rxp_album:
                     profile.clear_results()
                     set_status(translate('Masstagging',
-                        'Retrying search with album name: <b>%s</b>') %
-                        rxp_album)
+                                         'Retrying search with album name: <b>%s</b>') %
+                               rxp_album)
                     rxp_results = profile.search(changed_files)
                     results.extend(rxp_results)
                     profile.clear_results()
                 profile.results = results
-            except RetrievalError, e:
+            except RetrievalError as e:
                 if errors is None:
                     raise e
                 if errors(e, profile):
@@ -599,6 +612,7 @@ class MassTagProfile(object):
             profile.files = files
             yield profile.matched, profile.results, profile
 
+
 class Result(object):
     def __init__(self, info=None, tracks=None, tag_source=None):
         object.__init__(self)
@@ -619,7 +633,7 @@ class Result(object):
         self.__info = value
 
         if self.__tracks:
-            self.merged = map(lambda a: merge_track(a, value), self.__tracks)
+            self.merged = [merge_track(a, value) for a in self.__tracks]
         else:
             self.merged = []
 
@@ -638,7 +652,7 @@ class Result(object):
         if self.tag_source:
             try:
                 self.info, self.tracks = self.tag_source.retrieve(self.info)
-            except RetrievalError, e:
+            except RetrievalError as e:
                 if errors is None:
                     raise
                 if errors(e):
@@ -647,11 +661,11 @@ class Result(object):
                     return {}, []
             return self.info, self.tracks
         return {}, []
-            
+
 
 class TagSourceProfile(object):
     def __init__(self, files=None, tag_source=None, fields=None,
-            if_no_result=CONTINUE, replace_fields=None):
+                 if_no_result=CONTINUE, replace_fields=None):
 
         object.__init__(self)
 
@@ -695,7 +709,7 @@ class TagSourceProfile(object):
             else:
                 self.result = result
                 return self.result
-        except RetrievalError, e:
+        except RetrievalError as e:
             if errors is None:
                 raise
             if errors(e, self):
@@ -715,21 +729,20 @@ class TagSourceProfile(object):
         assert files
 
         files = split_by_field(files, *tag_source.group_by)
-        search_value = files.keys()[0]
-        self.results = map(lambda x: Result(*x),
-            tag_source.search(search_value, files[search_value]))
+        search_value = list(files.keys())[0]
+        self.results = [Result(*x) for x in tag_source.search(search_value, files[search_value])]
         for r in self.results:
             r.tag_source = self.tag_source
         return self.results
 
+
 if __name__ == '__main__':
-    import puddlestuff.puddletag
-    puddlestuff.puddletag.load_plugins()
-    from puddlestuff.tagsources import tagsources
+    from .. import puddletag
+
+    puddletag.load_plugins()
+    from ..tagsources import tagsources
+
     sources = dict((t.name, t) for t in tagsources)
     source = sources['Local TSource Plugin']()
-    source.applyPrefs([u'/mnt/multimedia/testlib'])
-    print source._dirs
-    
-    
-    
\ No newline at end of file
+    source.applyPrefs(['/mnt/multimedia/testlib'])
+    print(source._dirs)
