@@ -202,15 +202,15 @@ def help_menu(parent):
 
     doc_link = QAction(translate("Menus", 'Online &Documentation'),
                        parent)
-    connect(doc_link, lambda: open_url('http://puddletag.sf.net/docs.html'))
+    connect(doc_link, lambda: open_url('https://docs.puddletag.net/docs.html'))
 
-    forum_link = QAction(translate("Menus", '&Forum'), parent)
+    forum_link = QAction(translate("Menus", '&GitHub project'), parent)
     connect(forum_link,
-            lambda: open_url('http://sourceforge.net/apps/phpbb/puddletag'))
+            lambda: open_url('https://github.com/puddletag/puddletag'))
 
     issue_link = QAction(translate("Menus", '&Bug tracker'), parent)
     connect(issue_link,
-            lambda: open_url('http://code.google.com/p/puddletag/issues'))
+            lambda: open_url('https://github.com/puddletag/puddletag/issues'))
 
     about_icon = get_icon('help-about', QIcon())
     about = QAction(about_icon,
@@ -251,6 +251,12 @@ class PreviewLabel(QLabel):
         self._enabled = not self._enabled
         self.valueChanged.emit(self._enabled)
 
+def _openFilesFilterFilename(filename):
+    filename = os.path.abspath(filename)
+    if isinstance(filename, str):
+        filename = encode_fn(filename)
+    return filename
+
 
 class MainWin(QMainWindow):
     loadFiles = pyqtSignal(object, object, object, object, object, name='loadFiles')
@@ -448,7 +454,7 @@ class MainWin(QMainWindow):
         # not supported in PyQt5
         # filedlg.setResolveSymlinks(False) 
         filename = str(filedlg.getExistingDirectory(self,
-                                                    translate("Main Window", 'Import directory...'), dirname, QFileDialog.ShowDirsOnly))
+                                                    translate("Main Window", 'Import directory...'), dirname, QFileDialog.ShowDirsOnly|QFileDialog.DontUseNativeDialog))
         return filename
 
     def appendDir(self, filename=None):
@@ -562,6 +568,10 @@ class MainWin(QMainWindow):
 
         self.loadFiles.emit(None, [filename], append, None, None)
 
+    def openFiles(self, filenames, append=False):
+        filenames = map(_openFilesFilterFilename, filenames)
+        self.loadFiles.emit(None, filenames, append, None, None)
+
     def openPrefs(self):
         win = SettingsDialog(list(PuddleDock._controls.values()), self, status)
         win.show()
@@ -608,21 +618,21 @@ class MainWin(QMainWindow):
 
         mapping = {
             'VorbisComment':
-                {u'date': 'year',
+                {'date': 'year',
                  'tracknumber': 'track',
                  'musicbrainz_albumid': 'mbrainz_album_id',
                  'musicbrainz_artistid': 'mbrainz_artist_id',
                  'musicbrainz_trackid': 'mbrainz_track_id'},
             'MP4':
-                {u'MusicBrainz Track Id': 'mbrainz_track_id',
+                {'MusicBrainz Track Id': 'mbrainz_track_id',
                  'MusicBrainz Artist Id': 'mbrainz_artist_id',
                  'MusicBrainz Album Id': 'mbrainz_album_id'},
             'ID3':
-                {u'ufid:http://musicbrainz.org': 'mbrainz_track_id',
+                {'ufid:http://musicbrainz.org': 'mbrainz_track_id',
                  'MusicBrainz Album Id': 'mbrainz_album_id',
                  'MusicBrainz Artist Id': 'mbrainz_artist_id'},
             'APEv2':
-                {u'musicbrainz_albumid': 'mbrainz_album_id',
+                {'musicbrainz_albumid': 'mbrainz_album_id',
                  'musicbrainz_artistid': 'mbrainz_artist_id',
                  'musicbrainz_trackid': 'mbrainz_track_id'}}
 
@@ -741,6 +751,14 @@ class MainWin(QMainWindow):
                     if update:
                         lib_updates.append(update)
                     yield None
+                except PermissionError as e:
+                    failed_rows.append(row)
+                    filename = model.taginfo[row][PATH]
+                    m = rename_error_msg(e, filename)
+                    if row == rows[-1]:
+                        yield m, 1
+                    else:
+                        yield m, len(rows)
                 except EnvironmentError as e:
                     failed_rows.append(row)
                     filename = model.taginfo[row][PATH]
@@ -766,7 +784,6 @@ class MainWin(QMainWindow):
 
                 model.updateTable(failed_rows)
             return fin()
-
         return func, finished, rows
 
     def writeTags(self, tagiter, rows=None, previews=None):
