@@ -1099,7 +1099,10 @@ class TagModel(QAbstractTableModel):
             return
         else:
             artist = audio.get('artist', '')
-            undo_val = write(audio, tags, self.saveModification, justrename)
+            try:
+                undo_val = write(audio, tags, self.saveModification, justrename)
+            except PermissionError as e:
+                raise e
             if undo and undo_val:
                 self._addUndo(audio, undo_val)
 
@@ -1272,7 +1275,11 @@ class TagDelegate(QStyledItemDelegate):
     def eventFilter(self, editor, event):
         if event.type() == QEvent.KeyPress:
             if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-                if event.modifiers() == Qt.ShiftModifier:
+                if event.key() == Qt.Key_Return:
+                    shift_pressed = event.modifiers() == Qt.ShiftModifier
+                else:
+                    shift_pressed = event.modifiers() == Qt.ShiftModifier | Qt.KeypadModifier
+                if shift_pressed:
                     editor.returnPressed = SHIFT_RETURN
                 else:
                     editor.returnPressed = RETURN_ONLY
@@ -2087,6 +2094,17 @@ class TagTable(QTableView):
         self.selectionChanged()
 
     def reloadFiles(self, files=None):
+        previews = False
+        for z in self.model().taginfo:
+            if z.preview:
+                previews = True
+                break
+        if previews:
+            ret = QMessageBox.question(self, 'puddletag',
+                translate("Previews", 'There are unsaved changes pending. Do you want to discard and reload?'))
+            if ret != QMessageBox.Yes:
+                return
+
         self._restore = self.saveSelection()
         dirs = list(map(encode_fn, self.dirs))
         files = [z.filepath for z in self.model().taginfo if z.dirpath
